require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xdatabase',
  dscmeta_resource_friendly_name: 'xDatabase',
  dscmeta_resource_name: 'MSFT_xDatabase',
  dscmeta_module_name: 'xDatabase',
  dscmeta_module_version: '1.6.0.0',
  docs: 'The DSC xDatabase resource type.
         Automatically generated from version 1.6.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_sqlserver: {
      type: 'Optional[String]',
      desc: 'Sql Server Name',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlserverversion: {
      type: "Enum['2008-R2', '2012', '2014']",
      desc: 'Sql Server Version For DacFx',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_dacpacapplicationname: {
      type: 'Optional[String]',
      desc: 'DacPac Application Name for Registration',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dacpacpath: {
      type: 'Optional[String]',
      desc: 'Path to DacPac, if this is specified dacpac deployment is performed',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_bacpacpath: {
      type: 'Optional[String]',
      desc: 'Path to BacPac, if this is specified resore is performed',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_databasename: {
      type: 'String',
      desc: 'Name of the Database',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dacpacapplicationversion: {
      type: 'Optional[String]',
      desc: 'DacPac Application Version for Registration',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credentials: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials to Connect to the sql server',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Enum['Present', 'Absent']",
      desc: ' ',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
